@@ -83,27 +83,6 @@ $DEVICE_LIB_PATH=$ENV{'DEVICE_LIB_PATH'};
 $HIP_CLANG_HCC_COMPAT_MODE=$ENV{'HIP_CLANG_HCC_COMPAT_MODE'}; # HCC compatibility mode
 $HIP_COMPILE_CXX_AS_HIP=$ENV{'HIP_COMPILE_CXX_AS_HIP'} // "1";
 
-#---
-# Temporary directories
-my @tmpDirs = ();
-
-#---
-# Create a new temporary directory and return it
-sub get_temp_dir {
-    my $tmpdir = mkdtemp("/tmp/hipccXXXXXXXX");
-    push (@tmpDirs, $tmpdir);
-    return $tmpdir;
-}
-
-#---
-# Delete all created temporary directories
-sub delete_temp_dirs {
-    if (@tmpDirs) {
-        system ('rm -rf ' . join (' ', @tmpDirs));
-    }
-    return 0;
-}
-
 my $base_dir;
 BEGIN {
     $base_dir = dirname(Cwd::realpath(__FILE__) );
@@ -201,7 +180,6 @@ if ($HIP_PLATFORM eq "amd") {
     }
 } elsif ($HIP_PLATFORM eq "nvidia") {
     $CUDA_PATH=$ENV{'CUDA_PATH'} // '/usr/local/cuda';
-    $HIP_INCLUDE_PATH = "$HIP_PATH/include";
     if ($verbose & 0x2) {
         print ("CUDA_PATH=$CUDA_PATH\n");
     }
@@ -209,6 +187,7 @@ if ($HIP_PLATFORM eq "amd") {
     $HIPCC=get_normalized_path("$CUDA_PATH/bin/nvcc");
     $HIPCXXFLAGS .= " -Wno-deprecated-gpu-targets ";
     $HIPCXXFLAGS .= " -isystem " . get_normalized_path("$CUDA_PATH/include");
+    $HIPCXXFLAGS .= " -isystem " . get_normalized_path("$HIP_PATH/include");
     $HIPCFLAGS .= " -isystem " . get_normalized_path("$CUDA_PATH/include");
 
     $HIPLDFLAGS = " -Wno-deprecated-gpu-targets -lcuda -lcudart -L" . get_normalized_path("$CUDA_PATH/lib64");
@@ -250,11 +229,12 @@ if($HIP_PLATFORM eq "nvidia"){
     if($ARGV[0] eq "--genco"){
         foreach $isaarg (@ARGV[1..$#ARGV]){
             $ISACMD .= " ";
-            # ignore --rocm-path=xxxx on nvcc nvidia platform
-            if ($isaarg !~ /--rocm-path/) {
-              $ISACMD .= $isaarg;
+            # use the headers from rocm-path or hip-path
+            if (($isaarg =~ /--rocm-path/) or ($isaarg =~ /--hip-path/)) {
+              my @header_path = split('=', $isaarg);
+              $ISACMD .= '-I' . $header_path[1] .'/include';
             } else {
-              print "Ignoring --rocm-path= on nvidia nvcc platform.\n";
+              $ISACMD .= $isaarg;
             }
         }
         if ($verbose & 0x1) {
@@ -642,7 +622,6 @@ if ($runCmd) {
     else {
          $CMD_EXIT_CODE = $? >> 8;
     }
-    $? or delete_temp_dirs ();
     exit($CMD_EXIT_CODE);
 }
 
