@@ -25,8 +25,8 @@ use Getopt::Long;
 use Cwd;
 use File::Basename;
 
-$HIP_BASE_VERSION_MAJOR = "5";
-$HIP_BASE_VERSION_MINOR = "5";
+$HIP_BASE_VERSION_MAJOR = "6";
+$HIP_BASE_VERSION_MINOR = "2";
 $HIP_BASE_VERSION_PATCH = "0";
 
 #---
@@ -61,6 +61,7 @@ sub can_run {
 }
 
 $isWindows =  ($^O eq 'MSWin32' or $^O eq 'msys');
+$doubleQuote = "\"";
 
 #
 # TODO: Fix rpath LDFLAGS settings
@@ -74,6 +75,10 @@ $isWindows =  ($^O eq 'MSWin32' or $^O eq 'msys');
 # ROCM_PATH is defined relative to HIP_PATH else it is hardcoded to /opt/rocm.
 #
 $HIP_PATH=$ENV{'HIP_PATH'} // dirname(Cwd::abs_path("$0/../")); # use parent directory of hipcc
+if ($isWindows and defined $ENV{'HIP_PATH'}) {
+  $HIP_PATH =~ s/^"(.*)"$/$1/;
+  $HIP_PATH =~ s/\\/\//g;
+}
 if (-e "$HIP_PATH/bin/rocm_agent_enumerator") {
     $ROCM_PATH=$ENV{'ROCM_PATH'} // "$HIP_PATH"; # use HIP_PATH
 }elsif (-e "$HIP_PATH/../bin/rocm_agent_enumerator") { # case for backward compatibility
@@ -82,12 +87,16 @@ if (-e "$HIP_PATH/bin/rocm_agent_enumerator") {
     $ROCM_PATH=$ENV{'ROCM_PATH'} // "/opt/rocm";
 }
 $CUDA_PATH=$ENV{'CUDA_PATH'} // '/usr/local/cuda';
+if ($isWindows and defined $ENV{'CUDA_PATH'}) {
+  $CUDA_PATH =~ s/^"(.*)"$/$1/;
+  $CUDA_PATH =~ s/\\/\//g;
+}
 
-# Windows has a different structure, all binaries are inside hip/bin
-if ($isWindows) {
+# Windows/Distro's have a different structure, all binaries are with hipcc
+if ($isWindows or -e "$HIP_PATH/bin/clang") {
     $HIP_CLANG_PATH=$ENV{'HIP_CLANG_PATH'} // "$HIP_PATH/bin";
 } else {
-    $HIP_CLANG_PATH=$ENV{'HIP_CLANG_PATH'} // "$ROCM_PATH/llvm/bin";
+    $HIP_CLANG_PATH=$ENV{'HIP_CLANG_PATH'} // "$ROCM_PATH/lib/llvm/bin";
 }
 # HIP_ROCCLR_HOME is used by Windows builds
 $HIP_ROCCLR_HOME=$ENV{'HIP_ROCCLR_HOME'};
@@ -120,9 +129,9 @@ if (defined $HIP_RUNTIME and $HIP_RUNTIME eq "rocclr" and !defined $HIP_ROCCLR_H
 }
 
 if (not defined $HIP_PLATFORM) {
-    if (can_run("$HIP_CLANG_PATH/clang++") or can_run("clang++")) {
+    if (can_run($doubleQuote . "$HIP_CLANG_PATH/clang++" . $doubleQuote) or can_run("amdclang++")) {
         $HIP_PLATFORM = "amd";
-    } elsif (can_run("$CUDA_PATH/bin/nvcc") or can_run("nvcc")) {
+    } elsif (can_run($doubleQuote . "$CUDA_PATH/bin/nvcc" . $doubleQuote) or can_run("nvcc")) {
         $HIP_PLATFORM = "nvidia";
         $HIP_COMPILER = "nvcc";
         $HIP_RUNTIME = "cuda";
@@ -150,7 +159,11 @@ if ($HIP_COMPILER eq "clang") {
 #---
 # Read .hipVersion
 my %hipVersion = ();
-parse_config_file("$hipvars::HIP_PATH/bin/.hipVersion", \%hipVersion);
+if ($isWindows) {
+    parse_config_file("$hipvars::HIP_PATH/bin/.hipVersion", \%hipVersion);
+} else {
+    parse_config_file("$hipvars::HIP_PATH/share/hip/version", \%hipVersion);
+}
 $HIP_VERSION_MAJOR = $hipVersion{'HIP_VERSION_MAJOR'} // $HIP_BASE_VERSION_MAJOR;
 $HIP_VERSION_MINOR = $hipVersion{'HIP_VERSION_MINOR'} // $HIP_BASE_VERSION_MINOR;
 $HIP_VERSION_PATCH = $hipVersion{'HIP_VERSION_PATCH'} // $HIP_BASE_VERSION_PATCH;
