@@ -276,7 +276,7 @@ float atomicAdd_system(float* address, float val) {
 }
 
 #if !defined(__HIPCC_RTC__)
-DEPRECATED("use atomicAdd instead")
+HIP_DEPRECATED("use atomicAdd instead")
 #endif // !defined(__HIPCC_RTC__)
 __device__
 inline
@@ -612,44 +612,44 @@ float atomicMin(float* addr, float val) {
 #if defined(__AMDGCN_UNSAFE_FP_ATOMICS__)
   return unsafeAtomicMin(addr, val);
 #else
-  #if __has_builtin(__hip_atomic_load) && \
-      __has_builtin(__hip_atomic_compare_exchange_strong)
+  typedef union u_hold {
+    float a;
+    unsigned int b;
+  } u_hold_t;
+  u_hold_t u{val};
+  bool neg_zero = 0x80000000U == u.b;
+
   float value = __hip_atomic_load(addr, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_AGENT);
   bool done = false;
-  while (!done && value > val) {
+  while (!done && (value > val || (neg_zero && value == 0.0f))) {
     done = __hip_atomic_compare_exchange_strong(addr, &value, val,
                __ATOMIC_RELAXED, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_AGENT);
   }
   return value;
-  #else
-  unsigned int *uaddr = (unsigned int *)addr;
-  unsigned int value = __atomic_load_n(uaddr, __ATOMIC_RELAXED);
-  bool done = false;
-  while (!done && __uint_as_float(value) > val) {
-    done = __atomic_compare_exchange_n(uaddr, &value, __float_as_uint(val), false,
-               __ATOMIC_RELAXED, __ATOMIC_RELAXED);
-  }
-  return __uint_as_float(value);
-  #endif
 #endif
 }
 
 __device__
 inline
-float atomicMin_system(float* address, float val) {
-  unsigned int* uaddr { reinterpret_cast<unsigned int*>(address) };
-  #if __has_builtin(__hip_atomic_load)
-    unsigned int tmp {__hip_atomic_load(uaddr, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_SYSTEM)};
-  #else
-    unsigned int tmp {__atomic_load_n(uaddr, __ATOMIC_RELAXED)};
-  #endif
-  float value = __uint_as_float(tmp);
+float atomicMin_system(float* addr, float val) {
+#if defined(__AMDGCN_UNSAFE_FP_ATOMICS__)
+  return unsafeAtomicMin(addr, val);
+#else
+  typedef union u_hold {
+    float a;
+    unsigned int b;
+  } u_hold_t;
+  u_hold_t u{val};
+  bool neg_zero = 0x80000000U == u.b;
 
-  while (val < value) {
-    value = atomicCAS_system(address, value, val);
+  float value = __hip_atomic_load(addr, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_SYSTEM);
+  bool done = false;
+  while (!done && (value > val || (neg_zero && value == 0.0f))) {
+    done = __hip_atomic_compare_exchange_strong(addr, &value, val,
+               __ATOMIC_RELAXED, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_SYSTEM);
   }
-
   return value;
+#endif
 }
 
 __device__
@@ -658,44 +658,44 @@ double atomicMin(double* addr, double val) {
 #if defined(__AMDGCN_UNSAFE_FP_ATOMICS__)
   return unsafeAtomicMin(addr, val);
 #else
-  #if __has_builtin(__hip_atomic_load) && \
-      __has_builtin(__hip_atomic_compare_exchange_strong)
+  typedef union u_hold {
+    double a;
+    unsigned long long b;
+  } u_hold_t;
+  u_hold_t u{val};
+  bool neg_zero = 0x8000000000000000ULL == u.b;
+
   double value = __hip_atomic_load(addr, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_AGENT);
   bool done = false;
-  while (!done && value > val) {
+  while (!done && (value > val || (neg_zero && value == 0.0)))  {
     done = __hip_atomic_compare_exchange_strong(addr, &value, val,
                __ATOMIC_RELAXED, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_AGENT);
   }
   return value;
-  #else
-  unsigned long long *uaddr = (unsigned long long *)addr;
-  unsigned long long value = __atomic_load_n(uaddr, __ATOMIC_RELAXED);
-  bool done = false;
-  while (!done && __longlong_as_double(value) > val) {
-    done = __atomic_compare_exchange_n(uaddr, &value, __double_as_longlong(val), false,
-               __ATOMIC_RELAXED, __ATOMIC_RELAXED);
-  }
-  return __longlong_as_double(value);
-  #endif
 #endif
 }
 
 __device__
 inline
-double atomicMin_system(double* address, double val) {
-  unsigned long long* uaddr { reinterpret_cast<unsigned long long*>(address) };
-  #if __has_builtin(__hip_atomic_load)
-    unsigned long long tmp {__hip_atomic_load(uaddr, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_SYSTEM)};
-  #else
-    unsigned long long tmp {__atomic_load_n(uaddr, __ATOMIC_RELAXED)};
-  #endif
-  double value = __longlong_as_double(tmp);
+double atomicMin_system(double* addr, double val) {
+#if defined(__AMDGCN_UNSAFE_FP_ATOMICS__)
+  return unsafeAtomicMin(addr, val);
+#else
+  typedef union u_hold {
+    double a;
+    unsigned long long b;
+  } u_hold_t;
+  u_hold_t u{val};
+  bool neg_zero = 0x8000000000000000ULL == u.b;
 
-  while (val < value) {
-    value = atomicCAS_system(address, value, val);
+  double value = __hip_atomic_load(addr, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_SYSTEM);
+  bool done = false;
+  while (!done && (value > val || (neg_zero && value == 0.0)))  {
+    done = __hip_atomic_compare_exchange_strong(addr, &value, val,
+               __ATOMIC_RELAXED, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_SYSTEM);
   }
-
   return value;
+#endif
 }
 
 __device__
@@ -856,44 +856,46 @@ float atomicMax(float* addr, float val) {
 #if defined(__AMDGCN_UNSAFE_FP_ATOMICS__)
   return unsafeAtomicMax(addr, val);
 #else
-  #if __has_builtin(__hip_atomic_load) && \
-      __has_builtin(__hip_atomic_compare_exchange_strong)
-  float value = __hip_atomic_load(addr, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_AGENT);
+  typedef union u_hold {
+    float a;
+    unsigned int b;
+  } u_hold_t;
+  u_hold_t u;
+
+  u.a = __hip_atomic_load(addr, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_AGENT);
+  bool neg_zero = 0x80000000U == u.b;
   bool done = false;
-  while (!done && value < val) {
-    done = __hip_atomic_compare_exchange_strong(addr, &value, val,
+  while (!done && (u.a < val || (neg_zero && val == 0.0f))) {
+    done = __hip_atomic_compare_exchange_strong(addr, &u.a, val,
                __ATOMIC_RELAXED, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_AGENT);
+    neg_zero = 0x80000000U == u.b;
   }
-  return value;
-  #else
-  unsigned int *uaddr = (unsigned int *)addr;
-  unsigned int value = __atomic_load_n(uaddr, __ATOMIC_RELAXED);
-  bool done = false;
-  while (!done && __uint_as_float(value) < val) {
-    done = __atomic_compare_exchange_n(uaddr, &value, __float_as_uint(val), false,
-               __ATOMIC_RELAXED, __ATOMIC_RELAXED);
-  }
-  return __uint_as_float(value);
-  #endif
+  return u.a;
 #endif
 }
 
 __device__
 inline
-float atomicMax_system(float* address, float val) {
-  unsigned int* uaddr { reinterpret_cast<unsigned int*>(address) };
-  #if __has_builtin(__hip_atomic_load)
-    unsigned int tmp {__hip_atomic_load(uaddr, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_SYSTEM)};
-  #else
-    unsigned int tmp {__atomic_load_n(uaddr, __ATOMIC_RELAXED)};
-  #endif
-  float value = __uint_as_float(tmp);
+float atomicMax_system(float* addr, float val) {
+#if defined(__AMDGCN_UNSAFE_FP_ATOMICS__)
+  return unsafeAtomicMax(addr, val);
+#else
+  typedef union u_hold {
+    float a;
+    unsigned int b;
+  } u_hold_t;
+  u_hold_t u;
 
-  while (value < val) {
-    value = atomicCAS_system(address, value, val);
+  u.a = __hip_atomic_load(addr, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_SYSTEM);
+  bool neg_zero = 0x80000000U == u.b;
+  bool done = false;
+  while (!done && (u.a < val || (neg_zero && val == 0.0f))) {
+    done = __hip_atomic_compare_exchange_strong(addr, &u.a, val,
+               __ATOMIC_RELAXED, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_SYSTEM);
+    neg_zero = 0x80000000U == u.b;
   }
-
-  return value;
+  return u.a;
+#endif
 }
 
 __device__
@@ -902,44 +904,46 @@ double atomicMax(double* addr, double val) {
 #if defined(__AMDGCN_UNSAFE_FP_ATOMICS__)
   return unsafeAtomicMax(addr, val);
 #else
-  #if __has_builtin(__hip_atomic_load) && \
-      __has_builtin(__hip_atomic_compare_exchange_strong)
-  double value = __hip_atomic_load(addr, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_AGENT);
+  typedef union u_hold {
+    double a;
+    unsigned long long b;
+  } u_hold_t;
+  u_hold_t u;
+
+  u.a = __hip_atomic_load(addr, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_AGENT);
+  bool neg_zero = 0x8000000000000000ULL == u.b;
   bool done = false;
-  while (!done && value < val) {
-    done = __hip_atomic_compare_exchange_strong(addr, &value, val,
+  while (!done && (u.a < val || (neg_zero && val == 0.0))) {
+    done = __hip_atomic_compare_exchange_strong(addr, &u.a, val,
                __ATOMIC_RELAXED, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_AGENT);
+    neg_zero = 0x8000000000000000ULL == u.b;
   }
-  return value;
-  #else
-  unsigned long long *uaddr = (unsigned long long *)addr;
-  unsigned long long value = __atomic_load_n(uaddr, __ATOMIC_RELAXED);
-  bool done = false;
-  while (!done && __longlong_as_double(value) < val) {
-    done = __atomic_compare_exchange_n(uaddr, &value, __double_as_longlong(val), false,
-               __ATOMIC_RELAXED, __ATOMIC_RELAXED);
-  }
-  return __longlong_as_double(value);
-  #endif
+  return u.a;
 #endif
 }
 
 __device__
 inline
-double atomicMax_system(double* address, double val) {
-  unsigned long long* uaddr { reinterpret_cast<unsigned long long*>(address) };
-  #if __has_builtin(__hip_atomic_load)
-    unsigned long long tmp {__hip_atomic_load(uaddr, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_SYSTEM)};
-  #else
-    unsigned long long tmp {__atomic_load_n(uaddr, __ATOMIC_RELAXED)};
-  #endif
-  double value = __longlong_as_double(tmp);
+double atomicMax_system(double* addr, double val) {
+#if defined(__AMDGCN_UNSAFE_FP_ATOMICS__)
+  return unsafeAtomicMax(addr, val);
+#else
+  typedef union u_hold {
+    double a;
+    unsigned long long b;
+  } u_hold_t;
+  u_hold_t u;
 
-  while (value < val) {
-      value = atomicCAS_system(address, value, val);
+  u.a = __hip_atomic_load(addr, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_SYSTEM);
+  bool neg_zero = 0x8000000000000000ULL == u.b;
+  bool done = false;
+  while (!done && (u.a < val || (neg_zero && val == 0.0))) {
+    done = __hip_atomic_compare_exchange_strong(addr, &u.a, val,
+               __ATOMIC_RELAXED, __ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_SYSTEM);
+    neg_zero = 0x8000000000000000ULL == u.b;
   }
-
-  return value;
+  return u.a;
+#endif
 }
 
 __device__
@@ -947,22 +951,13 @@ inline
 unsigned int atomicInc(unsigned int* address, unsigned int val)
 {
 #if defined(__gfx941__)
-  __device__
-  extern
-  unsigned int __builtin_amdgcn_atomic_inc(
-    unsigned int*,
-    unsigned int,
-    unsigned int,
-    unsigned int,
-    bool) __asm("llvm.amdgcn.atomic.inc.i32.p0i32");
-
   return hip_cas_expander<__ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_AGENT>(
     address,
     val,
     [](unsigned int& x, unsigned int y) { x = (x >= y) ? 0 : (x + 1); },
     [=]() {
     return
-      __builtin_amdgcn_atomic_inc(address, val, __ATOMIC_RELAXED, 1, false);
+      __builtin_amdgcn_atomic_inc32(address, val, __ATOMIC_RELAXED, "agent");
   });
 #else
     return __builtin_amdgcn_atomic_inc32(address, val, __ATOMIC_RELAXED, "agent");
@@ -975,22 +970,13 @@ inline
 unsigned int atomicDec(unsigned int* address, unsigned int val)
 {
 #if defined(__gfx941__)
-  __device__
-  extern
-  unsigned int __builtin_amdgcn_atomic_dec(
-    unsigned int*,
-    unsigned int,
-    unsigned int,
-    unsigned int,
-    bool) __asm("llvm.amdgcn.atomic.dec.i32.p0i32");
-
   return hip_cas_expander<__ATOMIC_RELAXED, __HIP_MEMORY_SCOPE_AGENT>(
     address,
     val,
     [](unsigned int& x, unsigned int y) { x = (!x || x > y) ? y : (x - 1); },
     [=]() {
     return
-      __builtin_amdgcn_atomic_dec(address, val, __ATOMIC_RELAXED, 1, false);
+      __builtin_amdgcn_atomic_dec32(address, val, __ATOMIC_RELAXED, "agent");
   });
 #else
   return __builtin_amdgcn_atomic_dec32(address, val, __ATOMIC_RELAXED, "agent");
@@ -1406,7 +1392,7 @@ float atomicAdd(float* address, float val)
 }
 
 #if !defined(__HIPCC_RTC__)
-DEPRECATED("use atomicAdd instead")
+HIP_DEPRECATED("use atomicAdd instead")
 #endif // !defined(__HIPCC_RTC__)
 __device__
 inline
