@@ -38,6 +38,7 @@ THE SOFTWARE.
 
 #if defined(__has_attribute)
     #if __has_attribute(ext_vector_type)
+        #define __HIP_USE_NATIVE_VECTOR__ 1
         #define __NATIVE_VECTOR__(n, T) T __attribute__((ext_vector_type(n)))
     #else
         #define __NATIVE_VECTOR__(n, T) T[n]
@@ -509,7 +510,11 @@ template <typename __T> struct is_scalar : public integral_constant<bool, __is_s
         __HOST_DEVICE__
         HIP_vector_type& operator+=(const HIP_vector_type& x) noexcept
         {
+#if __HIP_USE_NATIVE_VECTOR__
             data += x.data;
+#else
+            for (auto i = 0u; i != rank; ++i) data[i] += x.data[i];
+#endif
             return *this;
         }
         template<
@@ -525,7 +530,11 @@ template <typename __T> struct is_scalar : public integral_constant<bool, __is_s
         __HOST_DEVICE__
         HIP_vector_type& operator-=(const HIP_vector_type& x) noexcept
         {
+#if __HIP_USE_NATIVE_VECTOR__
             data -= x.data;
+#else
+            for (auto i = 0u; i != rank; ++i) data[i] -= x.data[i];
+#endif
             return *this;
         }
         template<
@@ -541,7 +550,11 @@ template <typename __T> struct is_scalar : public integral_constant<bool, __is_s
         __HOST_DEVICE__
         HIP_vector_type& operator*=(const HIP_vector_type& x) noexcept
         {
+#if __HIP_USE_NATIVE_VECTOR__
             data *= x.data;
+#else
+            for (auto i = 0u; i != rank; ++i) data[i] *= x.data[i];
+#endif
             return *this;
         }
 
@@ -570,7 +583,11 @@ template <typename __T> struct is_scalar : public integral_constant<bool, __is_s
         __HOST_DEVICE__
         HIP_vector_type& operator/=(const HIP_vector_type& x) noexcept
         {
+#if __HIP_USE_NATIVE_VECTOR__
             data /= x.data;
+#else
+            for (auto i = 0u; i != rank; ++i) data[i] /= x.data[i];
+#endif
             return *this;
         }
         template<
@@ -590,7 +607,11 @@ template <typename __T> struct is_scalar : public integral_constant<bool, __is_s
         HIP_vector_type operator-() const noexcept
         {
             auto tmp(*this);
+#if __HIP_USE_NATIVE_VECTOR__
             tmp.data = -tmp.data;
+#else
+            for (auto i = 0u; i != rank; ++i) tmp.data[i] = -tmp.data[i];
+#endif
             return tmp;
         }
 
@@ -601,7 +622,11 @@ template <typename __T> struct is_scalar : public integral_constant<bool, __is_s
         HIP_vector_type operator~() const noexcept
         {
             HIP_vector_type r{*this};
+#if __HIP_USE_NATIVE_VECTOR__
             r.data = ~r.data;
+#else
+            for (auto i = 0u; i != rank; ++i) r.data[i] = ~r.data[i];
+#endif
             return r;
         }
 
@@ -611,7 +636,11 @@ template <typename __T> struct is_scalar : public integral_constant<bool, __is_s
         __HOST_DEVICE__
         HIP_vector_type& operator%=(const HIP_vector_type& x) noexcept
         {
+#if __HIP_USE_NATIVE_VECTOR__
             data %= x.data;
+#else
+            for (auto i = 0u; i != rank; ++i) data[i] %= x.data[i];
+#endif
             return *this;
         }
 
@@ -621,7 +650,11 @@ template <typename __T> struct is_scalar : public integral_constant<bool, __is_s
         __HOST_DEVICE__
         HIP_vector_type& operator^=(const HIP_vector_type& x) noexcept
         {
+#if __HIP_USE_NATIVE_VECTOR__
             data ^= x.data;
+#else
+            for (auto i = 0u; i != rank; ++i) data[i] ^= x.data[i];
+#endif
             return *this;
         }
 
@@ -631,7 +664,11 @@ template <typename __T> struct is_scalar : public integral_constant<bool, __is_s
         __HOST_DEVICE__
         HIP_vector_type& operator|=(const HIP_vector_type& x) noexcept
         {
+#if __HIP_USE_NATIVE_VECTOR__
             data |= x.data;
+#else
+            for (auto i = 0u; i != rank; ++i) data[i] |= x.data[i];
+#endif
             return *this;
         }
 
@@ -641,7 +678,11 @@ template <typename __T> struct is_scalar : public integral_constant<bool, __is_s
         __HOST_DEVICE__
         HIP_vector_type& operator&=(const HIP_vector_type& x) noexcept
         {
+#if __HIP_USE_NATIVE_VECTOR__
             data &= x.data;
+#else
+            for (auto i = 0u; i != rank; ++i) data[i] &= x.data[i];
+#endif
             return *this;
         }
 
@@ -651,7 +692,11 @@ template <typename __T> struct is_scalar : public integral_constant<bool, __is_s
         __HOST_DEVICE__
         HIP_vector_type& operator>>=(const HIP_vector_type& x) noexcept
         {
+#if __HIP_USE_NATIVE_VECTOR__
             data >>= x.data;
+#else
+            for (auto i = 0u; i != rank; ++i) data[i] >>= x.data[i];
+#endif
             return *this;
         }
 
@@ -661,7 +706,11 @@ template <typename __T> struct is_scalar : public integral_constant<bool, __is_s
         __HOST_DEVICE__
         HIP_vector_type& operator<<=(const HIP_vector_type& x) noexcept
         {
+#if __HIP_USE_NATIVE_VECTOR__
             data <<= x.data;
+#else
+            for (auto i = 0u; i != rank; ++i) data[i] <<= x.data[i];
+#endif
             return *this;
         }
     };
@@ -764,10 +813,10 @@ template <typename __T> struct is_scalar : public integral_constant<bool, __is_s
     __HOST_DEVICE__
     inline
     constexpr
-    bool _hip_any_zero(const V& x, int n) noexcept
+    bool _hip_compare(const V& x, const V& y, int n) noexcept
     {
         return
-            (n == -1) ? true : ((x[n] == 0) ? false : _hip_any_zero(x, n - 1));
+            (n == -1) ? true : ((x[n] != y[n]) ? false : _hip_compare(x, y, n - 1));
     }
 
     template<typename T, unsigned int n>
@@ -777,7 +826,7 @@ template <typename __T> struct is_scalar : public integral_constant<bool, __is_s
     bool operator==(
         const HIP_vector_type<T, n>& x, const HIP_vector_type<T, n>& y) noexcept
     {
-        return _hip_any_zero(x.data == y.data, n - 1);
+        return _hip_compare(x.data, y.data, n - 1);
     }
     template<typename T, unsigned int n, typename U>
     __HOST_DEVICE__
@@ -1057,6 +1106,54 @@ template <typename __T> struct is_scalar : public integral_constant<bool, __is_s
         return HIP_vector_type<T, n>{x} <<= y;
     }
 
+    /*
+     * Map HIP_vector_type<U, rankU> to HIP_vector_type<T, rankT>
+     */
+    template <typename T, unsigned int rankT, typename U, unsigned int rankU>
+    __forceinline__ __HOST_DEVICE__ typename std::enable_if<(rankT == 1 && rankU >= 1),
+                                                            const HIP_vector_type<T, rankT>>::type
+    __hipMapVector(const HIP_vector_type<U, rankU>& u) {
+      return HIP_vector_type<T, rankT>(static_cast<T>(u.x));
+    };
+
+    template <typename T, unsigned int rankT, typename U, unsigned int rankU>
+    __forceinline__ __HOST_DEVICE__ typename std::enable_if<(rankT == 2 && rankU == 1),
+                                                            const HIP_vector_type<T, rankT>>::type
+    __hipMapVector(const HIP_vector_type<U, rankU>& u) {
+      return HIP_vector_type<T, rankT> (static_cast<T>(u.x), static_cast<T>(0));
+    };
+
+    template <typename T, unsigned int rankT, typename U, unsigned int rankU>
+    __forceinline__ __HOST_DEVICE__ typename std::enable_if<(rankT == 2 && rankU >= 2),
+                                                            const HIP_vector_type<T, rankT>>::type
+    __hipMapVector(const HIP_vector_type<U, rankU>& u) {
+      return HIP_vector_type<T, rankT> (static_cast<T>(u.x), static_cast<T>(u.y));
+    };
+
+    template <typename T, unsigned int rankT, typename U, unsigned int rankU>
+    __forceinline__ __HOST_DEVICE__ typename std::enable_if<(rankT == 4 && rankU == 1),
+                                                            const HIP_vector_type<T, rankT>>::type
+    __hipMapVector(const HIP_vector_type<U, rankU>& u) {
+      return HIP_vector_type<T, rankT> (static_cast<T>(u.x), static_cast<T>(0),
+                                       static_cast<T>(0), static_cast<T>(0));
+    };
+
+    template <typename T, unsigned int rankT, typename U, unsigned int rankU>
+    __forceinline__ __HOST_DEVICE__ typename std::enable_if<(rankT == 4 && rankU == 2),
+                                                            const HIP_vector_type<T, rankT>>::type
+    __hipMapVector(const HIP_vector_type<U, rankU>& u) {
+      return HIP_vector_type<T, rankT>(static_cast<T>(u.x), static_cast<T>(u.y),
+                                       static_cast<T>(0), static_cast<T>(0));
+    };
+
+    template <typename T, unsigned int rankT, typename U, unsigned int rankU>
+    __forceinline__ __HOST_DEVICE__ typename std::enable_if<(rankT == 4 && rankU == 4),
+                                                            const HIP_vector_type<T, rankT>>::type
+    __hipMapVector(const HIP_vector_type<U, rankU>& u) {
+      return HIP_vector_type<T, rankT> (static_cast<T>(u.x), static_cast<T>(u.y),
+                                       static_cast<T>(u.z), static_cast<T>(u.w));
+    };
+
     #define __MAKE_VECTOR_TYPE__(CUDA_name, T) \
         using CUDA_name##1 = HIP_vector_type<T, 1>;\
         using CUDA_name##2 = HIP_vector_type<T, 2>;\
@@ -2030,118 +2127,6 @@ typedef union {
     type r{x, y, z, w};                                                                            \
     return r;                                                                                      \
   }
-
-template<typename T, typename U>
-__HOST_DEVICE__
-__forceinline__
-typename std::enable_if<
-  sizeof(T) / sizeof(typename T::value_type) == 1 &&
-  sizeof(U) / sizeof(typename U::value_type) >= 1, T>::type
-mapElem(const U &u) {
-  T t;
-  t.x = static_cast<typename T::value_type>(u.x);
-  return t;
-}
-
-template<typename T, typename U>
-__HOST_DEVICE__
-__forceinline__
-typename std::enable_if<
-  sizeof(T) / sizeof(typename T::value_type) == 2 &&
-  sizeof(U) / sizeof(typename U::value_type) >= 2, T>::type
-mapElem(const U &u) {
-  T t;
-  t.x = static_cast<typename T::value_type>(u.x);
-  t.y = static_cast<typename T::value_type>(u.y);
-  return t;
-}
-
-template<typename T, typename U>
-__HOST_DEVICE__
-__forceinline__
-typename std::enable_if<
-  sizeof(T) / sizeof(typename T::value_type) == 3 &&
-  sizeof(U) / sizeof(typename U::value_type) >= 3, T>::type
-mapElem(const U &u) {
-  T t;
-  t.x = static_cast<typename T::value_type>(u.x);
-  t.y = static_cast<typename T::value_type>(u.y);
-  t.z = static_cast<typename T::value_type>(u.z);
-  return t;
-}
-
-template<typename T, typename U>
-__HOST_DEVICE__
-__forceinline__
-typename std::enable_if<
-  sizeof(T) / sizeof(typename T::value_type) == 4 &&
-  sizeof(U) / sizeof(typename U::value_type) >= 4, T>::type
-mapElem(const U &u) {
-  T t;
-  t.x = static_cast<typename T::value_type>(u.x);
-  t.y = static_cast<typename T::value_type>(u.y);
-  t.z = static_cast<typename T::value_type>(u.z);
-  t.w = static_cast<typename T::value_type>(u.w);
-  return t;
-}
-
-template<typename T, typename U>
-__HOST_DEVICE__
-__forceinline__
-typename std::enable_if<
-        std::is_same<T, char>::value ||
-        std::is_same<T, unsigned char>::value ||
-        std::is_same<T, short>::value ||
-        std::is_same<T, unsigned short>::value ||
-        std::is_same<T, int>::value ||
-        std::is_same<T, unsigned int>::value ||
-        std::is_same<T, float>::value, const T>::type
-mapFrom(const U &u) {
-  union {
-    U u;
-    T t;
-  } d = { u };
-  return d.t;
-}
-
-template<typename T, typename U>
-__HOST_DEVICE__
-__forceinline__
-typename std::enable_if<
-        (sizeof(T) == sizeof(typename T::value_type)) ||
-        std::is_same<typename T::value_type, int>::value ||
-        std::is_same<typename T::value_type, unsigned int>::value ||
-        std::is_same<typename T::value_type, float>::value, const T>::type
-mapFrom(const U &u) {
-  union {
-    U u;
-    T t;
-  } d = { u };
-  return d.t;
-}
-
-template<typename T, typename U>
-__HOST_DEVICE__
-__forceinline__
-typename std::enable_if<
-        (sizeof(T) > sizeof(typename T::value_type)) && (
-        std::is_same<typename T::value_type, char>::value ||
-        std::is_same<typename T::value_type, unsigned char>::value ||
-        std::is_same<typename T::value_type, short>::value ||
-        std::is_same<typename T::value_type, unsigned short>::value), const T>::type
-mapFrom(const U &u) {
-  union {
-    U u;
-    int4 i4;
-    uint4 u4;
-  } d = { u };
-  if(std::is_signed<typename T::value_type>::value) {
-    return mapElem<T>(d.i4) ;
-  } else {
-    return mapElem<T>(d.u4);
-  }
-}
-
 #else
 #define DECLOP_MAKE_ONE_COMPONENT(comp, type)                                                      \
   static inline __HOST_DEVICE__ type make_##type(comp x) {                                         \
